searchData={"items":[{"type":"behaviour","title":"Kungfuig","doc":"The behaviour defining the dynamic config provider.\n\n`Kungfuig` provides a plugagble drop-in support for live configurations.\n\n#","ref":"Kungfuig.html"},{"type":"behaviour","title":"Examples - Kungfuig","doc":"Kungfuig.Supervisor.start_link()\n    Kungfuig.config()\n    #⇒ %{env: %{kungfuig: []}, system: %{}}\n\n    Kungfuig.config(:env)\n    #⇒ %{kungfuig: []}\n\n    Application.put_env(:kungfuig, :foo, 42)\n    Kungfuig.config(:env)\n    #⇒ %{kungfuig: [foo: 42]}\n\nThe configuration is frequently updated.","ref":"Kungfuig.html#module-examples"},{"type":"function","title":"Kungfuig.__struct__/0","doc":"The config that is manages by this behaviour is a simple map","ref":"Kungfuig.html#__struct__/0"},{"type":"function","title":"Kungfuig.config/2","doc":"","ref":"Kungfuig.html#config/2"},{"type":"callback","title":"Kungfuig.start_link/1","doc":"","ref":"Kungfuig.html#c:start_link/1"},{"type":"callback","title":"Kungfuig.update_config/1","doc":"The actual implementation to update the config","ref":"Kungfuig.html#c:update_config/1"},{"type":"type","title":"Kungfuig.callback/0","doc":"The callback to be used for subscibing to config updates.\n\nMight be an anonymous function, an `{m, f}` tuple accepting a single argument,\nor a process identifier accepting `call`, `cast` or simple message (`:info`.)","ref":"Kungfuig.html#t:callback/0"},{"type":"type","title":"Kungfuig.config/0","doc":"The config map to be updated and exposed through callback","ref":"Kungfuig.html#t:config/0"},{"type":"type","title":"Kungfuig.option/0","doc":"The option that can be passed to `start_link/1` function","ref":"Kungfuig.html#t:option/0"},{"type":"type","title":"Kungfuig.t/0","doc":"The `start_link/1` function wrapping the `GenServer.start_link/3`","ref":"Kungfuig.html#t:t/0"},{"type":"behaviour","title":"Kungfuig.Backend","doc":"The scaffold for the backend watching the external config source.","ref":"Kungfuig.Backend.html"},{"type":"callback","title":"Kungfuig.Backend.get/1","doc":"The implementation of the call to remote that retrieves the data","ref":"Kungfuig.Backend.html#c:get/1"},{"type":"callback","title":"Kungfuig.Backend.key/0","doc":"The key this particular config would be stored under, defaults to module name","ref":"Kungfuig.Backend.html#c:key/0"},{"type":"callback","title":"Kungfuig.Backend.report/1","doc":"The implementation of error reporting","ref":"Kungfuig.Backend.html#c:report/1"},{"type":"callback","title":"Kungfuig.Backend.transform/1","doc":"The transformer that converts the retrieved data to internal representation","ref":"Kungfuig.Backend.html#c:transform/1"},{"type":"behaviour","title":"Kungfuig.Validator","doc":"Generic validator encapsulation.\n\nDefault implementation is\n  [`NimbleOptions`](https://github.com/dashbitco/nimble_options).","ref":"Kungfuig.Validator.html"},{"type":"callback","title":"Kungfuig.Validator.doc/1","doc":"Generates a documentation for expected options","ref":"Kungfuig.Validator.html#c:doc/1"},{"type":"callback","title":"Kungfuig.Validator.validate/1","doc":"Validates given options with the schema, provided by the implementation of this behaviour","ref":"Kungfuig.Validator.html#c:validate/1"},{"type":"extras","title":"Kungfuig","doc":"# ![Kungfuig](https://raw.githubusercontent.com/kantox/kungfuig/master/stuff/kungfuig-48x48.png) Kungfuig    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/kantox/kungfuig/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/kantox/kungfuig/workflows/Dialyzer/badge.svg)","ref":"readme.html"},{"type":"extras","title":"Intro - Kungfuig","doc":"Live config supporting many different backends.\n\n**Kungfuig** (_pronounced:_ [ˌkʌŋˈfig]) provides an easy way to plug\nlive configuration into everything.\n\nIt provides backends for `env` and `system` and supports custom backends.","ref":"readme.html#intro"},{"type":"extras","title":"Installation - Kungfuig","doc":"```elixir\ndef deps do\n  [\n    {:kungfuig, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Using - Kungfuig","doc":"**Kungfuig** is the easy way to read the external configuration from sources that are not controlled by the application using it, such as _Redis_, or _Database_.\n\nHere is the example of backend implementation for the config read from external _MySQL_.\n\n```elixir\ndefmodule MyApp.Kungfuig.MySQL do\n  @moduledoc false\n\n  use Kungfuig.Backend\n\n  @impl Kungfuig.Backend\n  def get(_meta) do\n    with {:ok, host} <- System.fetch_env(\"MYSQL_HOST\"),\n         {:ok, db} <- System.fetch_env(\"MYSQL_DB\"),\n         {:ok, user} <- System.fetch_env(\"MYSQL_USER\"),\n         {:ok, pass} <- System.fetch_env(\"MYSQL_PASS\"),\n         {:ok, pid} when is_pid(pid) <-\n           MyXQL.start_link(hostname: host, database: db, username: user, password: pass),\n         result <- MyXQL.query!(pid, \"SELECT * FROM some_table\") do\n      GenServer.stop(pid)\n\n      result =\n        result.rows\n        |> Flow.from_enumerable()\n        |> Flow.map(fn [_, field1, field2, _, _] -> {field1, field2} end)\n        |> Flow.partition(key: &elem(&1, 0))\n        |> Flow.reduce(fn -> %{} end, fn {field1, field2}, acc ->\n          Map.update(\n            acc,\n            String.to_existing_atom(field1),\n            [field2],\n            &[field2 | &1]\n          )\n        end)\n\n      Logger.info(\"Loaded #{Enum.count(result)} values from \" <> host)\n\n      {:ok, result}\n    else\n      :error ->\n        Logger.warn(\"Skipped reconfig, one of MYSQL_{HOST,DB,USER,PASS} is missing\")\n        :ok\n\n      error ->\n        Logger.error(\"Reconfiguring failed. Error: \" <> inspect(error))\n        {:error, error}\n    end\n  end\nend\n```","ref":"readme.html#using"},{"type":"extras","title":"Testing - Kungfuig","doc":"Simply implement a stub returning an expected config and you are all set.\n\n```elixir\ndefmodule MyApp.Kungfuig.Stub do\n  @moduledoc false\n\n  use Kungfuig.Backend\n\n  @impl Kungfuig.Backend\n  def get(_meta), do: %{foo: :bar, baz: [42]}\nend\n```","ref":"readme.html#testing"},{"type":"extras","title":"Changelog - Kungfuig","doc":"- **`1.0.0`** — modern Elixir v1.16\n- **`0.4.4`** — fix a bug with hardcoded names (`Supervisor` and `Blender`)\n- **`0.4.2`** — allow `imminent: true` option to `Kungfuig.Backend`\n- **`0.4.0`** — allow named `Kungfuig.Supervisor` (thanks @vbroskas)\n- **`0.3.0`** — allow validation through `NimbleOptions` (per backend and global)\n- **`0.2.0`** — scaffold for backends + several callbacks (and the automatic one for `Blender`)\n\n## [Documentation](https://hexdocs.pm/kungfuig)","ref":"readme.html#changelog"}],"content_type":"text/markdown"}